# frozen_string_literal: true

module ComplexField
  module PersonIndexer
    def generate_solr_document
      super.tap do |solr_doc|
        index_person(solr_doc)
      end
    end

    def index_person(solr_doc)
      begin
        creators = object.complex_person.map { |c| (c.first_name + c.last_name).reject(&:blank?).join(' ') }
      rescue
        creators = []
      end
      begin
        creators << object.complex_person.map { |c| c.name.reject(&:blank?) }
      rescue
        creators ||= []
      end
      creators = creators.flatten.uniq.reject(&:blank?)
      return solr_doc unless creators
      solr_doc[Solrizer.solr_name('complex_person', :stored_searchable)] = creators
      solr_doc[Solrizer.solr_name('complex_person', :facetable)] = creators
      solr_doc[Solrizer.solr_name('complex_person', :displayable)] = object.complex_person.to_json
      object.complex_person.each do |c|
        # index creator by role
        person_name = c.name.reject(&:blank?)
        person_name = (c.first_name + c.last_name).reject(&:blank?).join(' ') if person_name.blank?
        label = 'other'
        roles = c.role.reject(&:blank?)
        if roles.present? and roles.first
          begin
            label = RoleService.new.label(roles.first)
            label = label.downcase.tr(' ', '_')
          rescue
            label = roles.first.downcase.tr(' ', '_')
          end
        end
        # complex_person by role as stored_searchable
        fld_name = Solrizer.solr_name("complex_person_#{label}", :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << person_name
        solr_doc[fld_name].flatten!
        # complex_person by role as facetable
        fld_name = Solrizer.solr_name("complex_person_#{label}", :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << person_name
        solr_doc[fld_name].flatten!
        # affiliation
        vals = c.affiliation.reject(&:blank?)
        fld_name = Solrizer.solr_name('complex_person_affiliation', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('complex_person_affiliation', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        # orcid
        vals = c.orcid.reject(&:blank?)
        fld_name = Solrizer.solr_name('complex_person_identifier', :symbol)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
      end
    end

    def self.person_facet_fields
      # solr fields that will be treated as facets
      fields = []
      roles = RoleService.new.select_all_options
      roles.each do |r|
        fields << Solrizer.solr_name("complex_person_#{r[1].downcase.tr(' ', '_')}", :facetable)
      end
      fields << Solrizer.solr_name('complex_person_affiliation', :facetable)
      fields
    end

    def self.person_search_fields
      # solr fields that will be used for a search
      fields = []
      fields << Solrizer.solr_name('complex_person', :stored_searchable)
      fields << Solrizer.solr_name('complex_person_identifier', :symbol)
      fields << Solrizer.solr_name('complex_person_affiliation', :stored_searchable)
      fields
    end

    def self.person_show_fields
      # solr fields that will be used to display results on the record page
      fields = []
      fields << Solrizer.solr_name('complex_person', :displayable)
      fields
    end
  end
end
